import os
from Screen import Screen
from Components.Button import Button
from Components.ServiceList import ServiceList
from Components.ActionMap import ActionMap
from Components.MovieList import MovieList
from Components.DiskInfo import DiskInfo
from Components.Label import Label
from Components.PluginComponent import plugins
from Components.Harddisk import harddiskmanager
from Components.config import config, ConfigSubsection, ConfigInteger, configfile
from Plugins.Plugin import PluginDescriptor

from Screens.MessageBox import MessageBox
from Screens.FixedMenu import FixedMenu
from Screens.ChoiceBox import ChoiceBox

from Tools.Directories import *
from Tools.BoundFunction import boundFunction

from enigma import eServiceReference, eServiceCenter, eTimer

#ralfk
config.movielist = ConfigSubsection()
config.movielist.moviesort = ConfigInteger(default=1)
config.movielist.listtype = ConfigInteger(default=1)

#3c5x9
global LAST_VIDEODIR
LAST_VIDEODIR = resolveFilename(SCOPE_HDD)

class ChannelContextMenu(FixedMenu):
	def __init__(self, session, csel, service):
		self.csel = csel
		self.service = service
		
		menu = [(_("back"), self.close), (_("delete..."), self.delete)]
		
		for p in plugins.getPlugins(PluginDescriptor.WHERE_MOVIELIST):
			menu.append((p.description, boundFunction(self.execPlugin, p)))
			
		#ralfk
		if config.movielist.moviesort.value == 0:
			menu.append(("sortieren nach Datum", self.sort_datum))
		else:
			menu.append(("sortieren alphabetisch", self.sort_alpha))
		
		menu.append(("Listenart orginal", self.movielist_org))
		menu.append(("Listenart kompakt mit Beschreibung", self.movielist_kompakt_descr))
		menu.append(("Listenart kompakt", self.movielist_kompakt))
		menu.append(("Listenart 1zeilig", self.movielist_1zeilig))
		
		FixedMenu.__init__(self, session, _("Movie Menu"), menu)
		self.skinName = "Menu"
		
	#ralfk
	def sort_datum(self):
		global saveflag
		saveflag = True
		config.movielist.moviesort.value = 1
		self.close()
		
	def sort_alpha(self):
		global saveflag
		saveflag = True
		config.movielist.moviesort.value = 0
		self.close()
		
	def movielist_org(self):
		global saveflag
		saveflag = True
		config.movielist.listtype.value = 0
		self.close()
		
	def movielist_kompakt_descr(self):
		global saveflag
		saveflag = True
		config.movielist.listtype.value = 1
		self.close()
		
	def movielist_kompakt(self):
		global saveflag
		saveflag = True
		config.movielist.listtype.value = 2
		self.close()
		
	def movielist_1zeilig(self):
		global saveflag
		saveflag = True
		config.movielist.listtype.value = 3
		self.close()
	

	def execPlugin(self, plugin):
		plugin(session=self.session, service=self.service)

	def delete(self):
		serviceHandler = eServiceCenter.getInstance()
		offline = serviceHandler.offlineOperations(self.service)
		info = serviceHandler.info(self.service)
		name = info and info.getName(self.service) or _("this recording")
		result = False
		if offline is not None:
			# simulate first
			if not offline.deleteFromDisk(1):
				result = True
		
		if result == True:
			self.session.openWithCallback(self.deleteConfirmed, MessageBox, _("Do you really want to delete %s?") % (name))
		else:
			self.session.openWithCallback(self.close, MessageBox, _("You cannot delete this!"), MessageBox.TYPE_ERROR)

	def deleteConfirmed(self, confirmed):
		if not confirmed:
			return self.close()
			
		serviceHandler = eServiceCenter.getInstance()
		offline = serviceHandler.offlineOperations(self.service)
		result = False
		if offline is not None:
			# really delete!
			if not offline.deleteFromDisk(0):
				result = True
		
		if result == False:
			self.session.openWithCallback(self.close, MessageBox, _("Delete failed!"), MessageBox.TYPE_ERROR)
		else:
			list = self.csel["list"].removeService(self.service)
			self.close()

class MovieSelection(Screen):
	def __init__(self, session, selectedmovie = None):
		Screen.__init__(self, session)
		#ralfk
		global saveflag
		saveflag = False
		
		self.tags = [ ]
		self.selected_tags = None
		#3c5x9 self.current_ref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + resolveFilename(SCOPE_HDD))
		
		global LAST_VIDEODIR

		self.current_ref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + LAST_VIDEODIR)
		
		self.movemode = False
		self.bouquet_mark_edit = False
		
		self.delayTimer = eTimer()
		self.delayTimer.timeout.get().append(self.updateHDDData)
		
		self["waitingtext"] = Label(_("Please wait... Loading list..."))
		
		self["list"] = MovieList(None)
		self.list = self["list"]
		self.selectedmovie = selectedmovie
		
		self["key_red"] = Button(_("All..."))
		self["key_green"] = Button("")
		self["key_yellow"] = Button("")
		self["key_blue"] = Button("")
		
		#self["okbutton"] = Button("ok", [self.channelSelected])
		
		#3c5x9 self["freeDiskSpace"] = DiskInfo(resolveFilename(SCOPE_HDD), DiskInfo.FREE, update=False)
		self["freeDiskSpace"] = self.diskinfo = DiskInfo(resolveFilename(SCOPE_HDD), DiskInfo.FREE, update=False)
		
		#3c5x9 self["actions"] = ActionMap(["OkCancelActions", "MovieSelectionActions", "ColorActions"],
		self["actions"] = ActionMap(["OkCancelActions", "MovieSelectionActions", "ColorActions","InfobarActions","HelpActions"],
			{
				"cancel": self.abort,
				"ok": self.movieSelected,
				"showEventInfo": self.showEventInformation,
				"contextMenu": self.doContext,
				
				"red": self.showAll,
				"green": self.showTagsFirst,
				"yellow": self.showTagsSecond,
				"blue": self.showTagsMenu,
				"showMovies": self.doPathSelect,
				"displayHelp": self.showhelp,
			})
		self["actions"].csel = self
		self.onShown.append(self.go)
		
		self.lengthTimer = eTimer()
		self.lengthTimer.timeout.get().append(self.updateLengthData)
		self.inited = False

	def showEventInformation(self):
		from Screens.EventView import EventViewSimple
		from ServiceReference import ServiceReference
		evt = self["list"].getCurrentEvent()
		if evt:
			self.session.open(EventViewSimple, evt, ServiceReference(self.getCurrent()))

	def go(self):
		if not self.inited:
		# ouch. this should redraw our "Please wait..."-text.
		# this is of course not the right way to do this.
			self.delayTimer.start(10, 1)
			self.inited=True

	def updateHDDData(self):
 		self.reloadList()
 		if self.selectedmovie is not None:
			self.moveTo()
		self["waitingtext"].instance.hide()

		self["freeDiskSpace"].update()

		self.updateTags()

	def updateLengthData(self):
		self.list.updateLengthOfIndex(self.lengthPosition)
		self.lengthPosition += 1
		if self.lengthPosition < self.lengthLength:
			self.lengthTimer.start(10, 1)

	def moveTo(self):
		self["list"].moveTo(self.selectedmovie)

	def getCurrent(self):
		return self["list"].getCurrent()

	def movieSelected(self):
		self.lengthTimer.stop()
		current = self.getCurrent()
		if current is not None:
			#ralfk
			self.saveconfig()
			self.close(current)

	def doContext(self):
		current = self.getCurrent()
		if current is not None:
			self.session.open(ChannelContextMenu, self, current)

	def abort(self):
		#ralfk
		self.saveconfig()
		self.close(None)
		
	#ralfk
	def saveconfig(self):
		global saveflag
		if saveflag == True:
			#os.system("wall saveconfig ok")
			config.movielist.moviesort.save()
			config.movielist.listtype.save()
			configfile.save()

	def getTagDescription(self, tag):
		# TODO: access the tag database
		return tag

	def updateTags(self):
		# get a list of tags available in this list
		self.tags = list(self["list"].tags)
		
		# by default, we do not display any filtering options
		self.tag_first = ""
		self.tag_second = ""
		
		# when tags are present, however, the first two are 
		# directly mapped to the second, third ("green", "yellow") buttons
		if len(self.tags) > 0:
			self.tag_first = self.getTagDescription(self.tags[0])
		
		if len(self.tags) > 1:
			self.tag_second = self.getTagDescription(self.tags[1])
		
		self["key_green"].text = self.tag_first
		self["key_yellow"].text = self.tag_second
		
		# the rest is presented in a list, available on the
		# fourth ("blue") button
		if len(self.tags) > 2:
			self["key_blue"].text = _("Other...")
		else:
			self["key_blue"].text = ""

	def reloadList(self):
		self["list"].reload(self.current_ref, self.selected_tags)
		self.lengthTimer.start(10, 1)
		self.lengthPosition = 0
		self.lengthLength = len(self["list"])
		
		title = _("Recorded files...")
		
		if self.selected_tags is not None:
			title += " - " + ','.join(self.selected_tags)
			
		title += "  " + LAST_VIDEODIR
		
		self.setTitle(title)

	def showAll(self):
		self.selected_tags = None
		self.reloadList()

	def showTagsN(self, n):
		if len(self.tags) < n:
			self.showTagWarning()
		else:
			print "select tag #%d, %s, %s" % (n, self.tags[n - 1], ','.join(self.tags))
			self.selected_tags = set([self.tags[n - 1]])
			self.reloadList()

	def showTagsFirst(self):
		self.showTagsN(1)

	def showTagsSecond(self):
		self.showTagsN(2)

	def tagChosen(self, tag):
		if tag is not None:
			self.selected_tags = set([tag[0]])
			self.reloadList()

	def showTagsMenu(self):
		if len(self.tags) < 3:
			self.showTagWarning()
		else:
			list = [(tag, self.getTagDescription(tag)) for tag in self.tags ]
			self.session.openWithCallback(self.tagChosen, ChoiceBox, title=_("Please select keyword to filter..."), list = list)

	def showTagWarning(self):
		# TODO
		self.session.open(MessageBox, _("You need to define some keywords first!\nPress the menu-key to define keywords.\nDo you want to define keywords now?"), MessageBox.TYPE_ERROR)

	def doPathSelect(self):
		# 3c5x9
		listpath = [("recorded Movies",resolveFilename(SCOPE_HDD))]
		# ralfk
		updir = ""
		if len(LAST_VIDEODIR.split('/')) >= 4:
			updir = '/'.join(LAST_VIDEODIR.split('/')[:-2]) + '/'
			if (updir == '/hdd/'):
				updir = "/media/hdd/"
			if len(updir.split('/')) >= 4:
				listpath.append(("..", updir))
			else:
				updir = ""
		if updir == "":
			listpath.append(("..List of Storage Devices", "mount"))
		
		# 3c5x9
		try:
			for i in os.listdir(LAST_VIDEODIR):
				if os.path.isdir(LAST_VIDEODIR+i):
					listpath.append((i.split("/")[-1],LAST_VIDEODIR+"%s/"%i))
			
			if updir != "":
				listpath.append(("..List of Storage Devices", "mount"))
		except Exception,e:
			print "Error while scanning subdirs ",e
		self.session.openWithCallback(self.PathSelected, ChoiceBox, title=_("Please select path for movies...\n\n"+LAST_VIDEODIR), list = listpath)
		
	def PathSelected(self,path):
		#3c5x9
		if path is not None:
			global LAST_VIDEODIR
			if path[1] != "mount":
				LAST_VIDEODIR = path[1]
				if (path[0] == "..") and (len(path[1].split('/')) == 4):
					self.mountSelected(path)	
				else:
					self.current_ref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + path[1])
					#if path[1] == resolveFilename(SCOPE_HDD):
					#	self["freeDiskSpace"].update()
					#else:
					#	self["freeDiskSpace"].path = LAST_VIDEODIR
					#	self["freeDiskSpace"].setText("Current Path: "+path[1])
			else:
				#os.system("wall %s" %path[0])
				#ralfk
				listpath = []
				for p in harddiskmanager.getMountedPartitions():
					if p.mountpoint == "/":
						listpath.append((p.description, p.mountpoint))
					else:
						listpath.append((p.description, p.mountpoint + "/"))
				self.session.openWithCallback(self.mountSelected, ChoiceBox, title=_("Please select mountpoint for movies...\n"), list = listpath)
				#self.doPathSelect2()
			self.reloadList()
			self.updateTags()
			
	def mountSelected(self,path):
		#ralfk
		if path is not None:
			global LAST_VIDEODIR
			#os.system("wall %s" %path[1])
			LAST_VIDEODIR = path[1]
			if path[0] == "..":
				listpath = [(path[1],LAST_VIDEODIR)]
			else:
				listpath = [(path[0],LAST_VIDEODIR)]
			
			try:
				for i in os.listdir(LAST_VIDEODIR):
					if os.path.isdir(LAST_VIDEODIR+i):
						listpath.append((i.split("/")[-1],LAST_VIDEODIR+"%s/"%i))
			except Exception,e:
				print "Error while scanning subdirs ",e
			self.session.openWithCallback(self.PathSelected2, ChoiceBox, title=_("Please select path for movies...\n\n"+LAST_VIDEODIR), list = listpath)
			
	def PathSelected2(self,path):
		global LAST_VIDEODIR
		if path is not None:
			LAST_VIDEODIR = path[1]
		self.current_ref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + LAST_VIDEODIR)
		self.reloadList()
		self.updateTags()
			
	def showhelp(self):
		text =        "Movieplayer Erweiterung von ralfk\n\n"
		text = text + "Mit der VIDEO-Taste kann das Verzeichnis gewechselt werden.\n\n"
		text = text + "Im contextMenu (DREAM-Taste) kann die Sortierart und die Listenart ausgewaehlt werden. Damit die Auswahl wirksam wird, muss die Movieliste geschlossen und mit der VIDEO-Taste wieder geoeffnet werden."
		self.session.open(MessageBox, text, type = MessageBox.TYPE_INFO ,timeout = 60)
		